@@ -3,21 +3,7 @@ package Geo::Coder::Multiple::Bing;
 use strict;
 use warnings;
 
-
-sub new {
-    my $class = shift;
-    my $args = shift;
-
-    my $self = {
-        GeoCoder    => $args->{geocoder},
-        daily_limit => $args->{daily_limit},
-        name        => $class,
-    };
-
-    bless $self, $class;
-
-    return( $self );
-};
+use base 'Geo::Coder::Multiple::Generic';
 
 
 sub geocode {
@@ -26,27 +12,23 @@ sub geocode {
 
     my $raw_reply = $self->{GeoCoder}->geocode( location => $location );
 
-    my $location_data = [];
+    my $Response = Geo::Coder::Multiple::Response->new( { location => $location } );
 
     foreach my $option ( @{$raw_reply->{Locations}} ) {
         my $tmp = {
             address     => $raw_reply->{Address}->{FormattedAddress},
             country     => $raw_reply->{Address}->{CountryRegion},
             longitude   => $option->{Coordinates}->{Longitude},
-            latitude   => $option->{Coordinates}->{Latitude},
-            geocoder    => 'bing',
+            latitude    => $option->{Coordinates}->{Latitude},
         };
 
-        push @{$location_data}, $tmp;
+        $Response->add_response( $tmp, 'bing' );
     };
 
-    return( $location_data );
+    return( $Response );
 };
 
 
-sub get_daily_limit { return( $_[0]->{daily_limit} ) };
-sub get_name { return( $_[0]->{name} ) };
-
 1;
 
 __END__
